%% Dyanmic elementary mode (dynEM) modelling
% Abel Folch-Fortuny, 2018
%
% MATLAB source code to execute dynamic elementary mode analysis (dynEMA)
% and dyamic elementary mode modelling regression discriminant analysis
% (dynEMR-DA)
%
% Additional MATLAB toolboxes NEEDED:
% - efmtool (http://www.csb.ethz.ch/tools/software/efmtool.html)
% - N-way toolbox (LINK)
% - Optimization toolbox for MATLAB (generic MATLAB package)
% 
% Comments:
% - Code is given for using dynEMR-DA with own data. Therefore, the
% commands below sould be changed/uncommented according to own data to
% allow calculations
% - As in the manuscript, dynEMR-DA is programmed testing one single dynEM
% each round for discrimination, which is usually met for some dynEMs.

%% 1. dynEMR-DA 
% Example data is used here to illustrate how the method works. It
% corresponds to a subset of the simulated data in the manuscript. 

clear,clc
load('ExampleFlux') % Example data
tic
[res]=dynEMRDA(flux3D,Y,EM,4)
toc
%% 2. Plot dynEMR-DA coefficients
% Using previous example data

% Calculate projection
[m,n,p]=size(flux3D);
flux2D=[];
for i=1:size(flux3D,1)
    flux2D=[flux2D;reshape(flux3D(i,:,:),n,p)'];
end
em_num=[8];
[Xrec,a2]=NNLS(flux2D,EM(:,em_num));

% Show plots
for t=1:p
    
    % Groups of experiments: valid if both have the same number of 
    % experiments, and all 1s are followed by all 0s in the dummy variable
    g0=[t:p:p*m/2];
    g1=[t+p*m/2:p:p*m];
    me0=mean(a2(g0,:));
    me1=mean(a2(g1,:));
    
    % Show one plot per time point
    figure
    b=bar([me1' me0'])
    title(['dynEM ', int2str(em_num)],'FontSize',12)
    ylabel(['time=',int2str(t)])
    xlabel('Coefficients')
    axis tight
    legend({'High glucose','Low glucose'},'Location','Best')
    set(b(1),'Facecolor', 'b');
    set(b(1),'EdgeColor','b');
    set(b(2),'Facecolor', 'r');
    set(b(2),'EdgeColor','r');
end

%% 3. Stoichiometric and Elementary Mode matrix
% In case S and EM are not available, they can be obtained with the
% following commands. A small network is used as example.

rformulas={
    '--> A'
    'A --> B'
    'B --> C'
    'C --> D'
    'C --> E'
    'C --> F'
    'F --> G'
    'D -->'
    'E -->'
    'G -->'
    }
mnet = CalculateFluxModes(rformulas)
S=mnet.stoich; %S matrix (metabolites x reactions)
EM=mnet.efms; %EM matrix (reactions x EM number)

%% 4. Flux or concentration data available
% In case flux data is available, name it as flux3D to execute section 1
% (dynEMR-DA). If not, fluxes can be obtained from concentation data as
% follows (uncomment lines below to execute, renaming your data in the 
% first line): 
% conc= %your concentration data. Cell array with experiments in different 
% % fields (row-wise) (time points x concentrations)
% % Matrix sizes
% [m,n]=size(conc);
% m=max(m,n);
% [n,p]=size(conc{1});
% % Obtain fluxes from concentrations
% for i=1:m
%     for j=1:n-1
%         dconc{i}(j,:)=conc{i}(j+1,:)-conc{i}(j,:);
%         flux{i}=FluxFromConc(dconc{i},mnet);
%     end
% end
% % Arrange 3D flux data matrix (experiments x fluxes x time points)
% for i=1:max(size(flux))
%     flux3D(i,:,:)=flux{i}';
% end

%% 5. Flux and concentration data NOT available

% An example is given in case no concentration/flux data is available. The
% example data connects to the network defined in section 1. However, the 
% data has no physical or biological meaning. It is just to illustrate how 
% data structures look like. In fact, all EMs might result significant. 
cA=[-3:0.25:0 zeros(1,6)];
cB=[-3 -3:0.25:0 zeros(1,5)];
cC=[-3*ones(1,2) -3:0.25:0 zeros(1,4)];
cD=[-3*ones(1,3) -3:0.25:0 zeros(1,3)];
cE=[-3*ones(1,4) -3:0.25:0 zeros(1,2)];
cF=[-3*ones(1,5) -3:0.25:0 zeros(1,1)];
cG=[-3*ones(1,6) -3:0.25:0];
C=[normpdf(cA)' normpdf(cB') normpdf(cC') normpdf(cD') normpdf(cE') normpdf(cF') normpdf(cG') ];
conc{1}=C*(1+0.2);
conc{2}=C*(1+0.1);
conc{3}=C*(1+0.3);
conc{4}=C*(1+0.15);
conc{5}=C*(1+0.25);
conc{6}=C*(4+0.2);
conc{7}=C*(4+0.1);
conc{8}=C*(4+0.3);
conc{9}=C*(4+0.15);
conc{10}=C*(4+0.25);

% Matrix sizes
[m,n]=size(conc);
m=max(m,n);
[n,p]=size(conc{1});

% Obtain fluxes from concentrations
for i=1:m
    for j=1:n-1
        dconc{i}(j,:)=conc{i}(j+1,:)-conc{i}(j,:);
        flux{i}=FluxFromConc(dconc{i},mnet);
    end
end

% Arrange 3D flux data matrix (experiments x fluxes x time points)
for i=1:max(size(flux))
    flux3D(i,:,:)=flux{i}';
end

%% 6. Discriminant variable
% The discriminant variable, and thus the structure of samples in the flux
% data, should follow the structure [ones(N,1);zeros(N,1)], as below (for 
% for the example data in 3. and 5.):

Y=[ones(5,1);zeros(5,1)];

